@@ -64,21 +64,20 @@ CU_TestInfo cs_tests[] = {
 
 CU_BOOL suite_cs_tests_enable(void)
 {
-	uint32_t asic_id;
-
 	if (amdgpu_device_initialize(drm_amdgpu[0], &major_version,
 					     &minor_version, &device_handle))
 		return CU_FALSE;
 
 	family_id = device_handle->info.family_id;
-	asic_id = device_handle->info.asic_id;
+    chip_id = device_handle->info.chip_external_rev;
+    chip_rev = device_handle->info.chip_rev;
 
 	if (amdgpu_device_deinitialize(device_handle))
 		return CU_FALSE;
 
 
 	if (family_id >= AMDGPU_FAMILY_RV || family_id == AMDGPU_FAMILY_SI ||
-		asic_is_arcturus(asic_id)) {
+		asic_is_gfx_pipe_removed(family_id, chip_id, chip_rev)) {
 		printf("\n\nThe ASIC NOT support UVD, suite disabled\n");
 		return CU_FALSE;
 	}
